﻿using System;

namespace Lambda表达式Demo
{
    class Program
    {
        /// <summary>
        /// Lambda 表达式使用演示
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            // Lambda表达式的演变过程
            // 下面是C# 1中创建委托实例的代码
            Func<string, int> delegatetest1 = new Func<string, int>(Callbackmethod);
            
            //                                  ↓
            // C# 2中用匿名方法来创建委托实例，此时就不需要额外定义回调方法Callbackmethod
            Func<string, int> delegatetest2 = delegate(string text)
            {
                return text.Length;
            };

            //                                  ↓
            // C# 3中使用Lambda表达式来创建委托实例
            Func<string, int> delegatetest3 = (string text) => text.Length;

            //                                  ↓
            // 可以省略参数类型string,把上面代码再简化为：
            Func<string, int> delegatetest4 = (text) => text.Length;

            //                                  ↓
            // 如果Lambda表达式只需一个参数，并且那个参数可以隐式指定类型时，
            // 此时可以把圆括号也省略,简化为：
            Func<string, int> delegatetest = text => text.Length;

            // 调用委托
            Console.WriteLine("使用Lambda表达式返回字符串的长度为： " + delegatetest("learning hard"));
            Console.Read();
        }

        /// <summary>
        /// 回调方法
        /// 如果使用了Lambda表达式和匿名函数，此方法就不需要额外定义
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        private static int Callbackmethod(string text)
        {
            return text.Length;
        }
    }
}
