﻿using System;

// 引用额外的命名空间
using System.Linq.Expressions;

namespace 表达式树Demo
{
    class Program
    {
        /// <summary>
        /// 表达式树的演示
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            #region 将Lambda表达式转换为表达式树演示
            // 将Lambda表达式转换为Express<T>的表达式树
            // 此时express不是可执行的代码，它现在是一个表达式树的数据结构
            Console.WriteLine("将Lambda表达式转化为表达式树的演示：");
            Expression<Func<int, int, int>> expression = (a, b) => a + b;

            // 获得表达式树的参数
            Console.WriteLine("参数1： {0},参数2：{1}", expression.Parameters[0],expression.Parameters[1]);

            // 既然叫做树，那肯定有左右节点
            // 获取表达式树的主体部分
            BinaryExpression body = (BinaryExpression)expression.Body;
            
            // 左节点,每个节点本身就是一个表达式对象
            ParameterExpression left = (ParameterExpression)body.Left;

            // 右节点
            ParameterExpression right = (ParameterExpression)body.Right;

            Console.WriteLine("表达式主体为：");
            Console.WriteLine(expression.Body);
            Console.WriteLine("表达式树左节点为：{0}{4} 节点类型为：{1}{4}{4} 表达式右节点为：{2}{4} 节点类型为：{3}{4}", left.Name, left.NodeType, right.Name, right.NodeType,Environment.NewLine);
            Console.Read();
            #endregion 

            #region 把表达式树转化回可执行代码

            // Compile方法生成Lambda表达式的委托
            Console.WriteLine("按下Enter键进入将表达式树转换为Lambda表达式的委托演示：");
            int result = expression.Compile()(2, 3);
            Console.WriteLine("调用Lambda表达式委托结果为：" + result);
            Console.ReadKey();
            #endregion
        }
    }
}
